@@ -141,11 +141,12 @@ class _PatchReader(object):
         if line is None or not line.startswith("@@ -"):
             return None
 
-        r = re.match("^@@ -(([0-9]+),)?([0-9]+) \+(([0-9]+),)?([0-9]+) @@", line)
+        r = re.match("^@@ -([0-9]+)(,([0-9]+))? \+([0-9]+)(,([0-9]+))? @@", line)
         if not r: raise PatchParserError("Unable to parse hunk header '%s'." % line)
-        srcpos = max(int(r.group(2)) - 1, 0) if r.group(2) else 0
-        dstpos = max(int(r.group(5)) - 1, 0) if r.group(5) else 0
-        srclines, dstlines = int(r.group(3)), int(r.group(6))
+        srcpos   = max(int(r.group(1)) - 1, 0)
+        dstpos   = max(int(r.group(4)) - 1, 0)
+        srclines = int(r.group(3)) if r.group(3) else 1
+        dstlines = int(r.group(6)) if r.group(6) else 1
         if srclines <= 0 and dstlines <= 0:
             raise PatchParserError("Empty hunk doesn't make sense.")
         self.read()
@@ -500,7 +501,7 @@ def generate_ifdef_patch(original, patched, ifdef):
     #
 
     with tempfile.NamedTemporaryFile(mode='w+') as diff:
-        exitcode = subprocess.call(["diff", "-u", original.name, patched.name],
+        exitcode = subprocess.call(["diff", "-U", "1", original.name, patched.name],
                                    stdout=diff, stderr=_devnull)
         if exitcode == 0:
             return None
@@ -612,10 +613,10 @@ def generate_ifdef_patch(original, patched, ifdef):
                 intermediate.write("\n")
 
             if len(srcdata) and len(dstdata):
-                intermediate.write("#if defined(%s)\n" % ifdef)
-                intermediate.write("\n".join(dstdata))
-                intermediate.write("\n#else  /* %s */\n" % ifdef)
+                intermediate.write("#if !defined(%s)\n" % ifdef)
                 intermediate.write("\n".join(srcdata))
+                intermediate.write("\n#else  /* %s */\n" % ifdef)
+                intermediate.write("\n".join(dstdata))
                 intermediate.write("\n#endif /* %s */\n" % ifdef)
 
             elif len(srcdata):
@@ -637,9 +638,9 @@ def generate_ifdef_patch(original, patched, ifdef):
             intermediate.write("\n")
         intermediate.flush()
 
-        # Now we can finally compute the diff between the patched file and our intermediate file
+        # Now we can finally compute the diff between the original file and our intermediate file
         diff = tempfile.NamedTemporaryFile(mode='w+')
-        exitcode = subprocess.call(["diff", "-u", patched.name, intermediate.name],
+        exitcode = subprocess.call(["git", "diff", "--no-index", original.name, intermediate.name],
                                    stdout=diff, stderr=_devnull)
         if exitcode != 1: # exitcode 0 cannot (=shouldn't) happen in this situation
             raise PatchDiffError("Failed to compute diff (exitcode %d)." % exitcode)
@@ -647,7 +648,11 @@ def generate_ifdef_patch(original, patched, ifdef):
         diff.flush()
         diff.seek(0)
 
-        # We expect this output format from 'diff', if this is not the case things might go wrong.
+        # We expect this output format from 'git diff', if this is not the case things might go wrong.
+        line = diff.readline()
+        assert line.startswith("diff --git ")
+        line = diff.readline()
+        assert line.startswith("index ")
         line = diff.readline()
         assert line.startswith("--- ")
         line = diff.readline()
@@ -909,23 +914,7 @@ if __name__ == "__main__":
 
             expected = ["@@ -1,9 +1,15 @@",
                         " line1();", " line2();", " line3();",
-                        "+#if defined(PATCHED)",
-                        " function(arg1, \\",
-                        "          new_arg2, \\",
-                        "          arg3);",
-                        "+#else  /* PATCHED */",
-                        "+function(arg1, \\",
-                        "+         arg2, \\",
-                        "+         arg3);",
-                        "+#endif /* PATCHED */",
-                        " line5();", " line6();", " line7();"]
-            diff = generate_ifdef_patch(source1, source2, "PATCHED")
-            lines = diff.read().rstrip("\n").split("\n")
-            self.assertEqual(lines, expected)
-
-            expected = ["@@ -1,9 +1,15 @@",
-                        " line1();", " line2();", " line3();",
-                        "+#if defined(PATCHED)",
+                        "+#if !defined(PATCHED)",
                         " function(arg1, \\",
                         "          arg2, \\",
                         "          arg3);",
@@ -935,6 +924,22 @@ if __name__ == "__main__":
                         "+         arg3);",
                         "+#endif /* PATCHED */",
                         " line5();", " line6();", " line7();"]
+            diff = generate_ifdef_patch(source1, source2, "PATCHED")
+            lines = diff.read().rstrip("\n").split("\n")
+            self.assertEqual(lines, expected)
+
+            expected = ["@@ -1,9 +1,15 @@",
+                        " line1();", " line2();", " line3();",
+                        "+#if !defined(PATCHED)",
+                        " function(arg1, \\",
+                        "          new_arg2, \\",
+                        "          arg3);",
+                        "+#else  /* PATCHED */",
+                        "+function(arg1, \\",
+                        "+         arg2, \\",
+                        "+         arg3);",
+                        "+#endif /* PATCHED */",
+                        " line5();", " line6();", " line7();"]
             diff = generate_ifdef_patch(source2, source1, "PATCHED")
             lines = diff.read().rstrip("\n").split("\n")
             self.assertEqual(lines, expected)
